-----------------------------------------
-- Politecnico di Torino
-- Date: 20 December 2020
-- Project: Counter subunit module ARRAY
-- Written by: Mohammadmahdi Mohammadi
-----------------------------------------


-----------------------------
---- LIBRARY DECLERATIN -----
-----------------------------
library IEEE;
use IEEE.std_logic_1164.all; 

----------------
---- ENTITY ----
----------------
entity CONVERTOR is
	Port 	(INPUT_ARRAY	:	In	std_logic_vector(86 downto 0);	-- Input array to be converted (8*1 + 3*2 + 1*3 + 1*5 + 1*11 + 2*32) = 97.
			 CLK			:	In	std_logic;						-- CLOCK
			 CONVERTOR_OUT	:	out std_logic_vector(15 downto 0)	-- CONVERTOR's output
			 );
end CONVERTOR;


----------------------
---- ARCHITECTURE ----
----------------------
architecture CONVERTOR_STR of CONVERTOR is

----------------------------
-- Components Declaration --
----------------------------
-- Based on selction procedure there will be 1, 2, 3, 5, 11 and 32 bit length inputs. which defines the needed components.
-- Counter SUBUNIT module
component CONVERTOR1 is
	Port 	(CONVERTOR_IN	:In	 std_logic;	   						-- INPUT VECTOR
			 CLK			:In	 std_logic;							-- CLOCK
			 CONVERTOR_OUT	:Out std_logic							-- SINGLE BIT OUTPUT
			 );
end component CONVERTOR1;

component CONVERTOR2 is
	Port 	(CONVERTOR_IN	:In	 std_logic_vector(1 downto 0);	   	-- INPUT VECTOR
			 CLK			:In	 std_logic;							-- CLOCK
			 CONVERTOR_OUT	:Out std_logic							-- SINGLE BIT OUTPUT
			 );
end component CONVERTOR2;

component CONVERTOR3 is
	Port 	(CONVERTOR_IN	:In	 std_logic_vector(2 downto 0);	   	-- INPUT VECTOR
			 CLK			:In	 std_logic;							-- CLOCK
			 CONVERTOR_OUT	:Out std_logic							-- SINGLE BIT OUTPUT
			 );
end component CONVERTOR3;

component CONVERTOR5 is
	Port 	(CONVERTOR_IN	:In	 std_logic_vector(4 downto 0);	   	-- INPUT VECTOR
			 CLK			:In	 std_logic;							-- CLOCK
			 CONVERTOR_OUT	:Out std_logic							-- SINGLE BIT OUTPUT
			 );
end component CONVERTOR5;

component CONVERTOR32 is
	Port 	(CONVERTOR_IN	:In	 std_logic_vector(31 downto 0);	   	-- INPUT VECTOR
			 CLK			:In	 std_logic;							-- CLOCK
			 CONVERTOR_OUT	:Out std_logic							-- SINGLE BIT OUTPUT
			 );
end component CONVERTOR32;

-------------------------
-- Signals Declaration --
-------------------------
-- Named based on chosen signals from design
-- 1BIT
signal fetch_en					:	std_logic;						--From Warp Scheduler
signal pipeline_warpunit_done	:	std_logic;						--From Warp Scheduler
signal warp_state_wr_en_out		:	std_logic;						--From Warp Scheduler
signal done						:	std_logic;						--From Warp Scheduler
signal pipeline_stall_in		:	std_logic;						--From Execution Unit
signal addr_imm_out				:	std_logic;						--From Execution Unit
signal pipeline_execute_done	:	std_logic;						--From Execution Unit
signal w32_in					:	std_logic;						--From Execution Unit
signal branch_exec_done			:	std_logic;						--From Execution Unit
--2BIT
signal warp_lane_id_out			:	std_logic_vector(1 downto 0);	--From Warp Scheduler
signal sm_type_out				:	std_logic_vector(1 downto 0);	--From Execution Unit
signal warp_lane_id_in			:	std_logic_vector(1 downto 0);	--From Execution Unit
--3BIT
signal mem_type_in				:	std_logic_vector(2 downto 0);	--From Execution Unit
--5BIT
signal warp_state_addr_out		:	std_logic_vector(4 downto 0);	--From Warp Scheduler
--32BIT
signal current_mask_out			:	std_logic_vector(31 downto 0);	--From Warp Scheduler
signal initial_mask_in			:	std_logic_vector(31 downto 0);	--From Execution Unit



begin

--INPUT TO SIGNAL CONNECTIONS
	fetch_en				<= INPUT_ARRAY(0);			
	pipeline_warpunit_done	<= INPUT_ARRAY(1);			
	warp_state_wr_en_out	<= INPUT_ARRAY(2);			
	done					<= INPUT_ARRAY(3);			
	pipeline_stall_in		<= INPUT_ARRAY(4);			
	addr_imm_out			<= INPUT_ARRAY(5);			
	pipeline_execute_done	<= INPUT_ARRAY(6);			
	w32_in					<= INPUT_ARRAY(7);			
	branch_exec_done		<= INPUT_ARRAY(8);			
	warp_lane_id_out		<= INPUT_ARRAY(10 downto 9);
	sm_type_out				<= INPUT_ARRAY(12 downto 11);
	warp_lane_id_in			<= INPUT_ARRAY(14 downto 13);
	mem_type_in				<= INPUT_ARRAY(17 downto 15);
	warp_state_addr_out		<= INPUT_ARRAY(22 downto 18);
	current_mask_out		<= INPUT_ARRAY(54 downto 23);
	initial_mask_in			<= INPUT_ARRAY(86 downto 55);


-- Single bit convertor subunit Instantiations
	fetch_en_conv: CONVERTOR1
	Port MAP (
		CONVERTOR_IN	=> fetch_en,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(0)
		);		
	
	pipeline_warpunit_done_conv: CONVERTOR1
	Port MAP (
		CONVERTOR_IN	=> pipeline_warpunit_done,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(1)
		);	

	warp_state_wr_en_out_conv: CONVERTOR1
	Port MAP (
		CONVERTOR_IN	=> warp_state_wr_en_out,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(2)
		);	

	done_conv: CONVERTOR1
	Port MAP (
		CONVERTOR_IN	=> done,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(3)
		);	
		
	pipeline_stall_in_conv: CONVERTOR1
	Port MAP (
		CONVERTOR_IN	=> pipeline_stall_in,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(4)
		);	
		
	addr_imm_out_conv: CONVERTOR1
	Port MAP (
		CONVERTOR_IN	=> addr_imm_out,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(5)
		);	

	pipeline_execute_done_conv: CONVERTOR1
	Port MAP (
		CONVERTOR_IN	=> pipeline_execute_done,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(6)
		);	
		
	w32_in_conv: CONVERTOR1
	Port MAP (
		CONVERTOR_IN	=> w32_in,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(7)
		);
		
	branch_exec_done_conv: CONVERTOR1
	Port MAP (
		CONVERTOR_IN	=> branch_exec_done,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(8)
		);
		
-- 2 bit convertor subunit Instantiations		
	warp_lane_cnt_conv: CONVERTOR2
	Port MAP (
		CONVERTOR_IN	=> warp_lane_id_out,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(9)
		);

	sm_type_out_conv: CONVERTOR2
	Port MAP (
		CONVERTOR_IN	=> sm_type_out,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(10)
		);

	warp_lane_id_in_conv: CONVERTOR2
	Port MAP (
		CONVERTOR_IN	=> warp_lane_id_in,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(11)
		);

-- 3 bit convertor subunit Instantiations		
	mem_type_in_conv: CONVERTOR3
	Port MAP (
		CONVERTOR_IN	=> mem_type_in,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(12)
		);

-- 5 bit convertor subunit Instantiations		
	warp_state_addr_out_conv: CONVERTOR5
	Port MAP (
		CONVERTOR_IN	=> warp_state_addr_out,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(13)
		);

		
-- 32 bit convertor subunit Instantiations		
	current_mask_out_conv: CONVERTOR32
	Port MAP (
		CONVERTOR_IN	=> current_mask_out,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(14)
		);
		
	initial_mask_in_conv: CONVERTOR32
	Port MAP (
		CONVERTOR_IN	=> initial_mask_in,
		CLK				=> CLK,
		CONVERTOR_OUT	=> CONVERTOR_OUT(15)
		);
		
	
end CONVERTOR_STR;



