-----------------------------------------
-- Politecnico di Torino
-- Date: 16 January 2021
-- Project: Signal convertor unit
-- Written by: Mohammadmahdi Mohammadi
-----------------------------------------

-----------------------------
---- LIBRARY DECLERATIN -----
-----------------------------
library IEEE;
use IEEE.std_logic_1164.all; 

----------------
---- ENTITY ----
----------------
entity CONVERTOR3 is
	Port 	(CONVERTOR_IN	:In	 std_logic_vector(2 downto 0);	   	-- INPUT VECTOR
			 CLK			:In	 std_logic;							-- CLOCK
			 CONVERTOR_OUT	:Out std_logic							-- SINGLE BIT OUTPUT
			 );
end CONVERTOR3;

----------------------
---- ARCHITECTURE ----
----------------------
architecture CONVERTOR3_STR of CONVERTOR3 is

----------------------------
-- Components Declaration --
----------------------------

-- D-FLIPFLOP COMPONENT
component FD is
	Port (D		:	In	std_logic;
		  CLK	:	In	std_logic;
		  RESET	:	In	std_logic;   -- active low reset 
		  EN	:	In	std_logic;
		  Q		:	Out	std_logic);
end component FD;

-------------------------
-- Signals Declaration --
-------------------------
signal	INPUT_s			:	std_logic_vector(2 downto 0):= (others => '0'); 	-- MAIN INPUT
signal	INPUT_DELAYED_s	:	std_logic_vector(2 downto 0):= (others => '0'); 	-- DELAYED INPUT
signal	VDD				:	std_logic := '1'; 									-- To be used for flipflops (TEMPORARY)
signal	CONVERTOR_OUT_s	:	std_logic := '0'; 	-- MAIN INPUT

begin  --CONVERTOR3_STR
	
	INPUT_s <= CONVERTOR_IN;
	
	-- One clock delay on input
	FLIPFLOP_ARRAY_GEN : for I in 2 downto 0 generate
	U1 : FD
		port map (INPUT_s(I), CLK, VDD, VDD, INPUT_DELAYED_s(I));
	end generate  FLIPFLOP_ARRAY_GEN;
	
	-- Comprator process
	Comprator : process(CLK)
	begin
		if CLK'event and CLK='1'	then
			if INPUT_s /= INPUT_DELAYED_s then
				CONVERTOR_OUT_s <= '1';
			elsif INPUT_s = INPUT_DELAYED_s then
				CONVERTOR_OUT_s <= '0';
			else
				CONVERTOR_OUT_s <= 'Z';
			end if;
		end if;
	end process;
	
	CONVERTOR_OUT <= CONVERTOR_OUT_s;	
	
end CONVERTOR3_STR;






