-----------------------------------------
-- Politecnico di Torino
-- Date: 18 DECEMBER 2020
-- Project: MONITOR UNIT DECODER DELIVERY Component
-- Written by: Mohammadmahdi Mohammadi
-----------------------------------------


-----------------------------
---- LIBRARY DECLERATIN -----
-----------------------------
library IEEE;
use IEEE.std_logic_1164.all; 
use ieee.numeric_std.all;
use work.gpgpu_package.all;



----------------
---- ENTITY ----
----------------
entity DECODER is
	Port 	(ADDRESS	:In	 std_logic_vector(3 downto 0);  --DECODER INPUT (ADDRESS)
			 COMMAND	:In	 instr_pmu_type; 				--DECODER INPUT (COMMAND)
			 RESET		:In  std_logic;						--Overal SYSTEM RESET acive high
			 COM_OUT0	:Out std_logic_vector(15 downto 0); --DECODER OUTPUT (0 BIT)
			 COM_OUT1	:Out std_logic_vector(15 downto 0));--DECODER OUTPUT (1 BIT)
end DECODER;


----------------------
---- ARCHITECTURE ----
----------------------
architecture DECODER_BEHAVIORAL of DECODER is

	signal zeros	:	std_logic_vector(15 downto 0) := (others => '0');
	signal ones		:	std_logic_vector(15 downto 0) := (others => '1');


begin
	

	
	process(COMMAND,ADDRESS,RESET) 		--DECODING process
	begin
		if (RESET='1') then
			COM_OUT0 <= zeros;
			COM_OUT1 <= ones;			-- RESET command to all subunits
		else
			COM_OUT0 <= zeros; 			-- default, DO NOTHING
			COM_OUT1 <= zeros; 			-- default, DO NOTHING
			
			if (COMMAND=PMURST) then	-- RESTART
				COM_OUT0(to_integer(unsigned(ADDRESS))) <= '0';
				COM_OUT1(to_integer(unsigned(ADDRESS)))<= '1';
			elsif (COMMAND=PMUIDL) then	-- IDLE
				COM_OUT0 (to_integer(unsigned(ADDRESS)))<= '1';
				COM_OUT1 (to_integer(unsigned(ADDRESS)))<= '1';
			elsif (COMMAND=PMUCNT) then	-- COUNTING
				COM_OUT0 (to_integer(unsigned(ADDRESS)))<= '1';
				COM_OUT1 (to_integer(unsigned(ADDRESS)))<= '0';
			else						-- NOP, DO NOTHING
				COM_OUT0 (to_integer(unsigned(ADDRESS)))<= '0';
				COM_OUT1 (to_integer(unsigned(ADDRESS)))<= '0';
			end if;
		end if;
		
	end process;

end DECODER_BEHAVIORAL;



----------------------
---- CONFIGURATION ---
----------------------
configuration CFG_DECODER_BEHAVIORAL of DECODER is
	for DECODER_BEHAVIORAL
  end for;
end configuration;	



