-----------------------------------------
-- Politecnico di Torino
-- Date: 18 December 2020
-- Project: PERFORMANCE MONITORING UNIT
-- Written by: Mohammadmahdi Mohammadi
-----------------------------------------


-----------------------------
---- LIBRARY DECLERATIN -----
-----------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use ieee.numeric_std.all; 
use work.gpgpu_package.all;


----------------
---- ENTITY ----
----------------
entity PM_UNIT is
	Port 	(ADDRESS	:	In	std_logic_vector(3 downto 0);	-- ADDRESS of intended subunit 
			 COMMAND	:	In	instr_pmu_type;					-- COMMAND signal from decoder
			 MONITOR_IN	:	In	std_logic_vector(86 downto 0);	-- Signals to be monitored
			 CLK		:	In	std_logic;	   					-- CLOCK 
			 RESET		:	In	std_logic;	   					-- OVERALL SYSTEM RESET Active High
			 MONITOR_OUT:	Out	std_logic_vector(31 downto 0));	-- MONITOR OUTPUT
end PM_UNIT;


----------------------
---- ARCHITECTURE ----
----------------------
architecture PM_UNIT_STRUCTURAL of PM_UNIT is

----------------------------
-- Components Declaration --
----------------------------

-- DECODER subunit module
component DECODER is
	Port 	(ADDRESS	:In	 std_logic_vector(3 downto 0);  --DECODER INPUT (ADDRESS)
			 COMMAND	:In	 instr_pmu_type;  				--DECODER INPUT (COMMAND)
			 RESET		:In  std_logic;						--Overal SYSTEM RESET acive high
			 COM_OUT0	:Out std_logic_vector(15 downto 0); --DECODER OUTPUT (0 BIT)
			 COM_OUT1	:Out std_logic_vector(15 downto 0));--DECODER OUTPUT (1 BIT)
end component DECODER;

-- Convertor module
component CONVERTOR is
	Port 	(INPUT_ARRAY	:	In	std_logic_vector(86 downto 0);	-- Input array to be converted (8*1 + 3*2 + 1*3 + 1*5 + 1*11 + 2*32) = 97.
			 CLK			:	In	std_logic;						-- CLOCK
			 CONVERTOR_OUT	:	out std_logic_vector(15 downto 0)	-- CONVERTOR's output
			 );
end component CONVERTOR;

-- Counter subunit ARRAY
component COUNTER_SUBUNIT_ARRAY is
	Port 	(COMMAND_0	:	In	std_logic_vector(15 downto 0);	   	-- State controller command FIRST BIT
			 COMMAND_1	:	In	std_logic_vector(15 downto 0);	   	-- State controller command NEXT BIT
			 MONITOR_IN	:	In	std_logic_vector(15 downto 0);		-- Signal to be monitored
			 CLK		:	In	std_logic;							-- CLOCK
			 COUNTER_OUT:	Out BUS_ARRAY							-- ARRAY's output
			 );
end component COUNTER_SUBUNIT_ARRAY;


-- 32 bit, 16 to 1 mux
component MUX_32_16to1 is
        generic (BUS_WIDTH 	: positive := 32;
                 SEL_BIT	: positive := 4);
        port    (I 	: in BUS_ARRAY;
                SEL : in std_logic_vector(SEL_BIT - 1 downto 0);
                O 	: out std_logic_vector(BUS_WIDTH - 1 downto 0));
end component MUX_32_16to1;


-------------------------
-- Signals Declaration --
-------------------------
signal	CONV_s		:	std_logic_vector(15 downto 0); 		-- CONVERTOR OUTPUT signal
signal	Z0_s		:	std_logic_vector(15 downto 0);		-- AND ARRAY GATE OUT1
signal	Z1_s		:	std_logic_vector(15 downto 0);		-- AND ARRAY GATE OUT2
signal	I_s			:	BUS_ARRAY;							-- MUX IN BUS

begin --PM_UNIT_STRUCTURAL

-- DECODER subunit module Instantiation
	DECODER_SUBUNIT_INS: DECODER
		Port map(
		ADDRESS			=> ADDRESS,		--DECODER INPUT (ADDRESS)
		COMMAND			=> COMMAND,		--DECODER INPUT (COMMAND)
		RESET			=> RESET,		--Overal SYSTEM RESET acive high
		COM_OUT0		=> Z0_s,		--DECODER OUTPUT (0 BIT)
		COM_OUT1		=> Z1_s			--DECODER OUTPUT (1 BIT)
		);


-- Convertor module Instantiation
	CONVERTOR_SUBUNIT_INS:	CONVERTOR
		port map(
		INPUT_ARRAY		=> MONITOR_IN,	-- Input array to be converted (8*1 + 3*2 + 1*3 + 1*5 + 1*11 + 2*32) = 97.
		CLK				=> CLK,			-- CLOCK
		CONVERTOR_OUT	=> CONV_s		-- CONVERTOR's output
		);

		
-- Counter subunit module Instantiation
	COUNTER_SUBUNIT_INS: COUNTER_SUBUNIT_ARRAY 	-- Generating the counters
		port map(
		COMMAND_0		=> Z0_s,		-- COMMAND first bit
		COMMAND_1		=> Z1_s,		-- COMMAND second bit
		MONITOR_IN  	=> CONV_s,		-- Cnverted inputs to be monitored
		CLK				=> CLK,			-- CLOCK			
		COUNTER_OUT		=> I_s			-- COUNTER SUBUNITS OUTPUT	
		);				

-- 32 bit, 16 to 1 mux Instantiation
	MUX_32_16to1_I: MUX_32_16to1
	port map(
		I				=> I_s,			-- MUX BUS INPUT
		SEL				=> ADDRESS,		-- ADDRESS provided by DECODER UNIT OF PROCESSOR
		O  				=> MONITOR_OUT	-- MONITORING UNIT OVERALL OUTPUT
		);	

end PM_UNIT_STRUCTURAL;   		-- END OF ARCHITECTURE.

	