-----------------------------------------
-- Politecnico di Torino
-- Date: 10 November 2020
-- Project: Generic Synchronus Counter
-- Written by: Mohammadmahdi Mohammadi
-----------------------------------------


-----------------------------
---- LIBRARY DECLERATIN -----
-----------------------------
library IEEE;
use IEEE.std_logic_1164.all; 


----------------
---- ENTITY ----
----------------
entity SYNC_COUNTER is
	Generic (NBITS	:	integer := 32);							-- Number of counter output bits
	Port 	(CLK	:	In	std_logic;	   						-- Clock 
			 Clear	:	In	std_logic;	   						-- Active low clear 
			 EN		:	In	std_logic;	   						-- Enables counting 
			 QOUT	:	Out	std_logic_vector(NBITS-1 downto 0));-- Counter output
end SYNC_COUNTER;


----------------------
---- ARCHITECTURE ----
----------------------
architecture SYNC_COUNT_STRUCTURAL of SYNC_COUNTER is

----------------------------
-- Components Declaration --
----------------------------

-- Toggle flip flop
component T_FF is
	Port (	T		:	In	std_logic;
			CLK		:	In	std_logic;
			Clear	:	In	std_logic;	-- Active high Clear 
			Q		:	Out	std_logic);
end component T_FF;

-- AND gate
component AND_GATE is
	Port (	X		:	In	std_logic;
			Y		:	In	std_logic;
			Z		:	Out	std_logic);
end component AND_GATE;

-------------------------
-- Signals Declaration --
-------------------------
signal	AND_OUT	:	std_logic_vector(NBITS-2 downto 0):= (others => '0'); -- Signal for and gates outputs
signal	QOUT_s	:	std_logic_vector(NBITS-1 downto 0):= (others => '0'); -- Counter output Signal 

begin


	SYNC_COUNTER: 						-- Generating the counter
	for I in 0 to NBITS-1 generate
	
	    FIRST_FF: if I=0 generate
			COUNTER0: T_FF port map
			(EN,CLK,Clear,QOUT_s(I));
		end generate FIRST_FF;
		
	    FIRST_AND_GATE: if I=0 generate
			AND0: AND_GATE port map
			(EN,QOUT_s(I),AND_OUT(I));
		end generate FIRST_AND_GATE;		
		
		NEXT_FFs : if I>0 generate
			COUNTERS: T_FF port map
			(AND_OUT(I-1),CLK,Clear,QOUT_S(I));
		end generate NEXT_FFs;
		
	    NEXT_AND_GATEs: if I<NBITS-1 and I>0 generate
			ANDs: AND_GATE port map
			(AND_OUT(I-1),QOUT_s(I),AND_OUT(I));
		end generate NEXT_AND_GATEs;		
		
	end generate SYNC_COUNTER;
	
	QOUT <=  QOUT_S; 					-- Signal assignment to output
	

end SYNC_COUNT_STRUCTURAL;




