module dp_regfile(clk, rst, addr_a, din_a, we_a, addr_b, din_b, we_b, dout_a, dout_b);

parameter RAM_SIZE 		= 1024;
parameter RAM_A_WIDTH 	= 10;
parameter RAM_D_WIDTH 	= 8;

input clk, rst;

input [RAM_A_WIDTH-1:0] addr_a, addr_b;
input [RAM_D_WIDTH-1:0] din_a, din_b;
input we_a, we_b;

output [RAM_D_WIDTH-1:0] dout_a, dout_b;
reg [RAM_D_WIDTH-1:0] dout_a, dout_b;

reg [RAM_D_WIDTH-1:0] memory [0:RAM_SIZE-1];

always @(posedge clk or posedge rst) begin
	if (rst = 1'b1) begin
	end else begin 
		if (we_a = 1'b1 and addr_a < RAM_SIZE)
			memory[addr_a] = din_a;
		if (we_b = 1'b1 and addr_b < RAM_SIZE)
		memory[addr_b] = din_b;
		end
	end

always @(addr_a, addr_b, we_a, we_b) begin
	if (we_b = 1'b1 and addr_a = addr_b)
		dout_a = din_b;
	else if (we_a = 1'b1)
		dout_a = din_a;
	else if (addr_a < RAM_SIZE)
		dout_a = memory[addr_a];
	else
		dout_a = 0;

	if (we_a = 1'b1 and addr_a = addr_b)
		dout_b = din_a;
	else if (we_b = 1'b1)
		dout_b = din_b;
	else if (addr_b < RAM_SIZE)
		dout_b = memory[addr_b];
	else
		dout_b = 0;
end

endmodule

