
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity TP_instructions is
	port(
		instruction_pointer_in : in  integer;
		num_instructions_out   : out integer;
		instruction_out        : out std_logic_vector(31 downto 0)
	);
end TP_instructions;

architecture arch of TP_instructions is
	constant TP_INSTRUCTIONS : integer := 74;

begin
	num_instructions_out <= TP_INSTRUCTIONS;

	process(instruction_pointer_in)
	begin
		case instruction_pointer_in is
			when 0 => instruction_out <= x"d0800209";   -- LOP.AND.U16 R1L, R0H, c[0x1][0x0];
when 1 => instruction_out <= x"00400780";
when 2 => instruction_out <= x"10004405";   -- MOV.U16 R0H, g [0x2].U16;
when 3 => instruction_out <= x"0023c780";
when 4 => instruction_out <= x"a0000409";   -- I2I.U32.U16 R2, R1L;
when 5 => instruction_out <= x"04000780";
when 6 => instruction_out <= x"10004209";   -- MOV.U16 R1L, g [0x1].U16;
when 7 => instruction_out <= x"0023c780";
when 8 => instruction_out <= x"a000000d";   -- I2I.U32.U16 R3, R0L;
when 9 => instruction_out <= x"04000780";
when 10 => instruction_out <= x"60014e01";   -- IMAD.U16 R0, g [0x7].U16, R0H, R2;
when 11 => instruction_out <= x"00208780";
when 12 => instruction_out <= x"60024c05";   -- IMAD.U16 R1, g [0x6].U16, R1L, R3;
when 13 => instruction_out <= x"0020c780";
when 14 => instruction_out <= x"30040001";   -- SHL R0, R0, 0x4;
when 15 => instruction_out <= x"c4100780";
when 16 => instruction_out <= x"20000001";   -- IADD R0, R0, R1;
when 17 => instruction_out <= x"04004780";
when 18 => instruction_out <= x"30010005";   -- SHL R1, R0, 0x1;
when 19 => instruction_out <= x"c4100780";
when 20 => instruction_out <= x"20018004";   -- IADD32 R1, R0, R1;
when 21 => instruction_out <= x"2101e804";   -- IADD32 R1, g [0x4], R1;
when 22 => instruction_out <= x"20018211";   -- IADD32I R4, R1, 0x1;
when 23 => instruction_out <= x"00000003";
when 24 => instruction_out <= x"2002820d";   -- IADD32I R3, R1, 0x2;
when 25 => instruction_out <= x"00000003";
when 26 => instruction_out <= x"d00e0209";   -- GLD.U8 R2, global14[R1];
when 27 => instruction_out <= x"80000780";
when 28 => instruction_out <= x"d00e0805";   -- GLD.U8 R1, global14[R4];
when 29 => instruction_out <= x"80000780";
when 30 => instruction_out <= x"d00e060d";   -- GLD.U8 R3, global14[R3];
when 31 => instruction_out <= x"80000780";
when 32 => instruction_out <= x"a0000809";   -- I2I.U32.U16.BEXT R2, R2L;
when 33 => instruction_out <= x"04008780";
when 34 => instruction_out <= x"a0000405";   -- I2I.U32.U16.BEXT R1, R1L;
when 35 => instruction_out <= x"04008780";
when 36 => instruction_out <= x"20000409";   -- IADD R2, R2, R1;
when 37 => instruction_out <= x"04004780";
when 38 => instruction_out <= x"a0000c05";   -- I2I.U32.U16.BEXT R1, R3L;
when 39 => instruction_out <= x"04008780";
when 40 => instruction_out <= x"20000209";   -- IADD.C0 R2, R1, R2;
when 41 => instruction_out <= x"040087c0";
when 42 => instruction_out <= x"102b8005";   -- MVI R1, -0x55555555;
when 43 => instruction_out <= x"0aaaaaab";
when 44 => instruction_out <= x"a0000409";   -- I2I.U32.S32 R2, |R2|;
when 45 => instruction_out <= x"04114780";
when 46 => instruction_out <= x"4003080d";   -- IMUL.U16.U16 R3, R2L, R1H;
when 47 => instruction_out <= x"00000780";
when 48 => instruction_out <= x"60020a0d";   -- IMAD.U16.C1 R3, R2H, R1L, R3;
when 49 => instruction_out <= x"0000c7d0";
when 50 => instruction_out <= x"30100611";   -- SHL R4, R3, 0x10;
when 51 => instruction_out <= x"c4100780";
when 52 => instruction_out <= x"3010060d";   -- SHR R3, R3, 0x10;
when 53 => instruction_out <= x"e4100780";
when 54 => instruction_out <= x"600209fd";   -- IMAD.U16.C2 o[0x7f], R2L, R1L, R4;
when 55 => instruction_out <= x"000107e8";
when 56 => instruction_out <= x"2100060d";   -- IADD R3 (C1.CARRY), R3, c[0x1][0x1];
when 57 => instruction_out <= x"04405880";
when 58 => instruction_out <= x"60030a05";   -- IMAD.U16 R1 (C3.TRUE), -R2H, R1H, R3;
when 59 => instruction_out <= x"0c00e780";
when 60 => instruction_out <= x"30010205";   -- SHR.S32 R1, R1, 0x1;
when 61 => instruction_out <= x"ec100780";
when 62 => instruction_out <= x"30008209";   -- IADD32I R2, -R1, 0x0;
when 63 => instruction_out <= x"00000003";
when 64 => instruction_out <= x"30020001";   -- SHL R0, R0, 0x2;
when 65 => instruction_out <= x"c4100780";
when 66 => instruction_out <= x"10000405";   -- MOV R1 (C0.SIGN), R2;
when 67 => instruction_out <= x"0403c980";
when 68 => instruction_out <= x"2000cc01";   -- IADD R0, g [0x6], R0;
when 69 => instruction_out <= x"04200780";
when 70 => instruction_out <= x"d00e0005";   -- GST.U32 global14[R0], R1;
when 71 => instruction_out <= x"a0c00781";
when 72 => instruction_out <= x"30000003";   -- RET
when 73 => instruction_out <= x"00000780";

			when others => null;
		end case;
	end process;

end arch;
