
-- use this file as the predefined for each app.

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

package pick_bench is

    type cmem_regs_type   is array (63 downto 0) of std_logic_vector(7 downto 0);
    type kernel_regs_type is array (15 downto 0) of std_logic_vector(31 downto 0);
	
	
    constant BENCH_CORES : integer := 16;
    constant BENCH_WARP_LANES : integer := 2;

    constant BENCH_APP : string := "TP";     -- vector_add (256 block - 8 warps)
    constant BENCH_APP_INST : string := "TP";

	-- parametros del kernel, dependen de la aplicacion, address and limit count
    constant BENCH_KREG0: std_logic_vector(31 downto 0) := x"00000000";
    constant BENCH_KREG1: std_logic_vector(31 downto 0) := x"00000100";
    constant BENCH_KREG2: std_logic_vector(31 downto 0) := x"00000200";
    constant BENCH_KREG3: std_logic_vector(31 downto 0) := x"00000008";
	
	
    constant BENCH_K_GRDX : std_logic_vector(15 downto 0) := x"0002";  	-- 1 (64 escrituras en total), ejecucion de cada grip y de los block en cada grid
    constant BENCH_K_GRDY : std_logic_vector(15 downto 0) := x"0001";  	-- 0 adimensional, no hay definicion del grid, minimo valor =1
    
    constant BENCH_K_BLKX : std_logic_vector(15 downto 0) := x"0020";	-- 20	-- 20=(32 memory write block access), 40=(64 mem write access)
    constant BENCH_K_BLKY : std_logic_vector(15 downto 0) := x"0001";  	-- fixed for now
    constant BENCH_K_BLKZ : std_logic_vector(15 downto 0) := x"0001";	-- fixed for now
	
	-- valores almacenados en la memoria constante, dependen de la aplicacion.
    constant BENCH_CMEMREG0 : std_logic_vector(7 downto 0) := x"ff";
    constant BENCH_CMEMREG1 : std_logic_vector(7 downto 0) := x"03";
    constant BENCH_CMEMREG2 : std_logic_vector(7 downto 0) := x"ff";
    constant BENCH_CMEMREG3 : std_logic_vector(7 downto 0) := x"03";

    constant BENCH_CMEM_PARAM_SIZE          : std_logic_vector(5 downto 0)  :=  "100000";  	 -- Constant memory limit depends on app.
    constant BENCH_KERNEL_PARAM_SIZE        : std_logic_vector(3 downto 0)  :=  x"4";
    constant BENCH_KERNEL_GPRS              : std_logic_vector(8 downto 0)  :=  "000100000"; -- 32 -- Dependiendo de la app. el minimo numero de GPRs debe ser igual al numero presentado.
    constant BENCH_KERNEL_SHMEM_SIZE        : std_logic_vector(31 downto 0) :=  x"0000002c";
    constant BENCH_BLOCKS_PER_CORE          : std_logic_vector(3 downto 0)  :=  x"1";  		 -- 8
 	
--    constant READ_DATA_SIZE : std_logic_vector(17 downto 0) := "001000100000100000";
    constant READ_DATA_SIZE : std_logic_vector(17 downto 0) := std_logic_vector(to_unsigned(319, 18));
	
    constant cmem_regs_default : cmem_regs_type := ( -- 63(3F)
        x"00", x"00", x"00", x"00",                  -- 60
        x"00", x"00", x"00", x"00",                  -- 56
        x"00", x"00", x"00", x"00",                  -- 52
        x"00", x"00", x"00", x"00",                  -- 48
        x"00", x"00", x"00", x"00",                  -- 44
        x"00", x"00", x"00", x"00",                  -- 40
        x"00", x"00", x"00", x"00",                  -- 36
        x"00", x"00", x"00", x"00",                  -- 32
        x"00", x"00", x"00", x"00",                  -- 28
        x"00", x"00", x"00", x"00",                  -- 24
        x"00", x"00", x"00", x"00",                  -- 20
        x"00", x"00", x"00", x"00",                  -- 16
        x"00", x"00", x"00", x"00",                  -- 12       
        x"00", x"00", x"00", x"00",                  -- 8     
        x"00", x"00", x"00", x"00",                  -- 4     
        x"00", x"00", x"03", x"ff");                 -- 0                                         

    -- ORIGINAL FOR TP:
    -- x"03", x"ff", x"03", x"ff"

    constant kernel_regs_default : kernel_regs_type := (     -- 15 (0F)
        x"00000000", x"00000000", x"00000000", x"00000000",  -- 12 
        x"00000000", x"00000000", x"00000000", x"00000000",  -- 8
        x"00000000", x"00000000", x"00000000", x"00000000",  -- 4
        x"00000000", x"00000040", x"00000200", x"00000000"); -- 0   	
		
		
		
	
end pick_bench;
