
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity TP_instructions is
	port(
		instruction_pointer_in : in  integer;
		num_instructions_out   : out integer;
		instruction_out        : out std_logic_vector(31 downto 0)
	);
end TP_instructions;

architecture arch of TP_instructions is
	constant TP_INSTRUCTIONS : integer := 340;

begin
	num_instructions_out <= TP_INSTRUCTIONS;

	process(instruction_pointer_in)
	begin
		case instruction_pointer_in is
			when 0 => instruction_out <= x"a0004e0d";   -- I2I.U32.U16 R3, g [0x7].U16;
when 1 => instruction_out <= x"04200780";
when 2 => instruction_out <= x"1000ce05";   -- MOV R1, g [0x7];
when 3 => instruction_out <= x"0423c780";
when 4 => instruction_out <= x"40030c09";   -- IMUL.U16.U16 R2, R3L, R1H;
when 5 => instruction_out <= x"00000780";
when 6 => instruction_out <= x"60020e09";   -- IMAD.U16 R2, R3H, R1L, R2;
when 7 => instruction_out <= x"00008780";
when 8 => instruction_out <= x"30100409";   -- SHL R2, R2, 0x10;
when 9 => instruction_out <= x"c4100780";
when 10 => instruction_out <= x"60020c05";   -- IMAD.U16 R1, R3L, R1L, R2;
when 11 => instruction_out <= x"00008780";
when 12 => instruction_out <= x"30040205";   -- SHL R1, R1, 0x4;
when 13 => instruction_out <= x"c4100780";
when 14 => instruction_out <= x"2000ce09";   -- IADD R2, g [0x7], R1;
when 15 => instruction_out <= x"04204780";
when 16 => instruction_out <= x"203f841d";   -- IADD32I R7, R2, 0xffffffff;
when 17 => instruction_out <= x"0fffffff";
when 18 => instruction_out <= x"d0800205";   -- LOP.AND.U16 R0H, R0H, c[0x1][0x0];
when 19 => instruction_out <= x"00400780";
when 20 => instruction_out <= x"41102c19";   -- IMUL32I.U16.U16 R6, g [0x6].U16, 0x10;
when 21 => instruction_out <= x"00000003";
when 22 => instruction_out <= x"30010ffd";   -- ISET.S32.C0 o[0x7f], R7, R1, LT;
when 23 => instruction_out <= x"6c0047c8";
when 24 => instruction_out <= x"a0000029";   -- I2I.U32.U16 R10, R0L;
when 25 => instruction_out <= x"04000780";
when 26 => instruction_out <= x"a0000209";   -- I2I.U32.U16 R2, R0H;
when 27 => instruction_out <= x"04000780";
when 28 => instruction_out <= x"10098003";   -- BRA C0.NE, 0x4c0;
when 29 => instruction_out <= x"00000280";
when 30 => instruction_out <= x"1100f010";   -- MOV32 R4, g [0x8];
when 31 => instruction_out <= x"1100ee00";   -- MOV32 R0, g [0x7];
when 32 => instruction_out <= x"40010814";   -- IMUL32.U16.U16 R5, R2L, R0H;
when 33 => instruction_out <= x"40090820";   -- IMUL32.U16.U16 R8, R2L, R4H;
when 34 => instruction_out <= x"60000a15";   -- IMAD.U16 R5, R2H, R0L, R5;
when 35 => instruction_out <= x"00014780";
when 36 => instruction_out <= x"60080a21";   -- IMAD.U16 R8, R2H, R4L, R8;
when 37 => instruction_out <= x"00020780";
when 38 => instruction_out <= x"30100a15";   -- SHL R5, R5, 0x10;
when 39 => instruction_out <= x"c4100780";
when 40 => instruction_out <= x"30101021";   -- SHL R8, R8, 0x10;
when 41 => instruction_out <= x"c4100780";
when 42 => instruction_out <= x"3004042d";   -- SHL R11, R2, 0x4;
when 43 => instruction_out <= x"c4100780";
when 44 => instruction_out <= x"60000801";   -- IMAD.U16 R0, R2L, R0L, R5;
when 45 => instruction_out <= x"00014780";
when 46 => instruction_out <= x"60080825";   -- IMAD.U16 R9, R2L, R4L, R8;
when 47 => instruction_out <= x"00020780";
when 48 => instruction_out <= x"30060415";   -- SHL R5, R2, 0x6;
when 49 => instruction_out <= x"c4100780";
when 50 => instruction_out <= x"200b9410";   -- IADD32 R4, R10, R11;
when 51 => instruction_out <= x"20008200";   -- IADD32 R0, R1, R0;
when 52 => instruction_out <= x"20000c09";   -- IADD R2, R6, R9;
when 53 => instruction_out <= x"04024780";
when 54 => instruction_out <= x"20388a15";   -- IADD32I R5, R5, 0x438;
when 55 => instruction_out <= x"00000043";
when 56 => instruction_out <= x"00020805";   -- R2A A1, R4, 0x2;
when 57 => instruction_out <= x"c0000780";
when 58 => instruction_out <= x"20009400";   -- IADD32 R0, R10, R0;
when 59 => instruction_out <= x"20029408";   -- IADD32 R2, R10, R2;
when 60 => instruction_out <= x"00000a0d";   -- R2A A3, R5;
when 61 => instruction_out <= x"c0000780";
when 62 => instruction_out <= x"00021409";   -- R2A A2, R10, 0x2;
when 63 => instruction_out <= x"c0000780";
when 64 => instruction_out <= x"30020001";   -- SHL R0, R0, 0x2;
when 65 => instruction_out <= x"c4100780";
when 66 => instruction_out <= x"30020409";   -- SHL R2, R2, 0x2;
when 67 => instruction_out <= x"c4100780";
when 68 => instruction_out <= x"3006d021";   -- SHL R8, g [0x8], 0x6;;
when 69 => instruction_out <= x"c4300780";
when 70 => instruction_out <= x"1000f82d";   -- MOV R11, R124;
when 71 => instruction_out <= x"0403c780";
when 72 => instruction_out <= x"2100e814";   -- IADD32 R5, g [0x4], R0;
when 73 => instruction_out <= x"2102ea10";   -- IADD32 R4, g [0x5], R2;
when 74 => instruction_out <= x"d00e0a09";   -- GLD.U32 R2, global14[R5];
when 75 => instruction_out <= x"80c00780";
when 76 => instruction_out <= x"d00e0801";   -- GLD.U32 R0, global14[R4];
when 77 => instruction_out <= x"80c00780";
when 78 => instruction_out <= x"04021c01";   -- R2G.U32.U32 g[A1+0x10e], R2;
when 79 => instruction_out <= x"e4208780";
when 80 => instruction_out <= x"04001c01";   -- R2G.U32.U32 g[A1+0xe], R0;
when 81 => instruction_out <= x"e4200780";
when 82 => instruction_out <= x"861ffe03";   -- BAR.ARV.WAIT b0, 0xfff;
when 83 => instruction_out <= x"00000000";
when 84 => instruction_out <= x"1900fc00";   -- MOV32 R0, g [A2+0xe];
when 85 => instruction_out <= x"1d00e008";   -- MOV32 R2, g [A3+0x0];
when 86 => instruction_out <= x"40050031";   -- IMUL.U16.U16 R12, R0L, R2H;
when 87 => instruction_out <= x"00000780";
when 88 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 89 => instruction_out <= x"00030780";
when 90 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 91 => instruction_out <= x"c4100780";
when 92 => instruction_out <= x"60040031";   -- IMAD.U16 R12, R0L, R2L, R12;
when 93 => instruction_out <= x"00030780";
when 94 => instruction_out <= x"1c00c201";   -- MOV R0, g [A3+0x1];
when 95 => instruction_out <= x"0423c780";
when 96 => instruction_out <= x"1800fc09";   -- MOV R2, g [A2+0x1e];
when 97 => instruction_out <= x"0423c780";
when 98 => instruction_out <= x"200b982c";   -- IADD32 R11, R12, R11;
when 99 => instruction_out <= x"40050030";   -- IMUL32.U16.U16 R12, R0L, R2H;
when 100 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 101 => instruction_out <= x"00030780";
when 102 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 103 => instruction_out <= x"c4100780";
when 104 => instruction_out <= x"d8017011";   -- ADA A4, A2, 0xb8;
when 105 => instruction_out <= x"20000780";
when 106 => instruction_out <= x"60040031";   -- IMAD.U16 R12, R0L, R2L, R12;
when 107 => instruction_out <= x"00030780";
when 108 => instruction_out <= x"1c00c401";   -- MOV R0, g [A3+0x2];
when 109 => instruction_out <= x"0423c780";
when 110 => instruction_out <= x"1000c009";   -- MOV R2, g [A4+0x0];
when 111 => instruction_out <= x"0423c784";
when 112 => instruction_out <= x"200b982c";   -- IADD32 R11, R12, R11;
when 113 => instruction_out <= x"40050030";   -- IMUL32.U16.U16 R12, R0L, R2H;
when 114 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 115 => instruction_out <= x"00030780";
when 116 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 117 => instruction_out <= x"c4100780";
when 118 => instruction_out <= x"60040031";   -- IMAD.U16 R12, R0L, R2L, R12;
when 119 => instruction_out <= x"00030780";
when 120 => instruction_out <= x"1c00c601";   -- MOV R0, g [A3+0x3];
when 121 => instruction_out <= x"0423c780";
when 122 => instruction_out <= x"1000e009";   -- MOV R2, g [A4+0x10];
when 123 => instruction_out <= x"0423c784";
when 124 => instruction_out <= x"200b982c";   -- IADD32 R11, R12, R11;
when 125 => instruction_out <= x"40050030";   -- IMUL32.U16.U16 R12, R0L, R2H;
when 126 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 127 => instruction_out <= x"00030780";
when 128 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 129 => instruction_out <= x"c4100780";
when 130 => instruction_out <= x"d8027011";   -- ADA A4, A2, 0x138;
when 131 => instruction_out <= x"20000780";
when 132 => instruction_out <= x"60040031";   -- IMAD.U16 R12, R0L, R2L, R12;
when 133 => instruction_out <= x"00030780";
when 134 => instruction_out <= x"1c00c801";   -- MOV R0, g [A3+0x4];
when 135 => instruction_out <= x"0423c780";
when 136 => instruction_out <= x"1000c009";   -- MOV R2, g [A4+0x0];
when 137 => instruction_out <= x"0423c784";
when 138 => instruction_out <= x"200b982c";   -- IADD32 R11, R12, R11;
when 139 => instruction_out <= x"40050030";   -- IMUL32.U16.U16 R12, R0L, R2H;
when 140 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 141 => instruction_out <= x"00030780";
when 142 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 143 => instruction_out <= x"c4100780";
when 144 => instruction_out <= x"60040031";   -- IMAD.U16 R12, R0L, R2L, R12;
when 145 => instruction_out <= x"00030780";
when 146 => instruction_out <= x"1c00ca01";   -- MOV R0, g [A3+0x5];
when 147 => instruction_out <= x"0423c780";
when 148 => instruction_out <= x"1000e009";   -- MOV R2, g [A4+0x10];
when 149 => instruction_out <= x"0423c784";
when 150 => instruction_out <= x"200b982c";   -- IADD32 R11, R12, R11;
when 151 => instruction_out <= x"40050030";   -- IMUL32.U16.U16 R12, R0L, R2H;
when 152 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 153 => instruction_out <= x"00030780";
when 154 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 155 => instruction_out <= x"c4100780";
when 156 => instruction_out <= x"d8037011";   -- ADA A4, A2, 0x1b8;
when 157 => instruction_out <= x"20000780";
when 158 => instruction_out <= x"60040031";   -- IMAD.U16 R12, R0L, R2L, R12;
when 159 => instruction_out <= x"00030780";
when 160 => instruction_out <= x"1c00cc01";   -- MOV R0, g [A3+0x6];
when 161 => instruction_out <= x"0423c780";
when 162 => instruction_out <= x"1000c009";   -- MOV R2, g [A4+0x0];
when 163 => instruction_out <= x"0423c784";
when 164 => instruction_out <= x"200b982c";   -- IADD32 R11, R12, R11;
when 165 => instruction_out <= x"40050030";   -- IMUL32.U16.U16 R12, R0L, R2H;
when 166 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 167 => instruction_out <= x"00030780";
when 168 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 169 => instruction_out <= x"c4100780";
when 170 => instruction_out <= x"60040031";   -- IMAD.U16 R12, R0L, R2L, R12;
when 171 => instruction_out <= x"00030780";
when 172 => instruction_out <= x"1c00ce01";   -- MOV R0, g [A3+0x7];
when 173 => instruction_out <= x"0423c780";
when 174 => instruction_out <= x"1000e009";   -- MOV R2, g [A4+0x10];
when 175 => instruction_out <= x"0423c784";
when 176 => instruction_out <= x"200b982c";   -- IADD32 R11, R12, R11;
when 177 => instruction_out <= x"40050030";   -- IMUL32.U16.U16 R12, R0L, R2H;
when 178 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 179 => instruction_out <= x"00030780";
when 180 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 181 => instruction_out <= x"c4100780";
when 182 => instruction_out <= x"d8047011";   -- ADA A4, A2, 0x238;
when 183 => instruction_out <= x"20000780";
when 184 => instruction_out <= x"60040031";   -- IMAD.U16 R12, R0L, R2L, R12;
when 185 => instruction_out <= x"00030780";
when 186 => instruction_out <= x"1c00d001";   -- MOV R0, g [A3+0x8];
when 187 => instruction_out <= x"0423c780";
when 188 => instruction_out <= x"1000c009";   -- MOV R2, g [A4+0x0];
when 189 => instruction_out <= x"0423c784";
when 190 => instruction_out <= x"200b982c";   -- IADD32 R11, R12, R11;
when 191 => instruction_out <= x"40050030";   -- IMUL32.U16.U16 R12, R0L, R2H;
when 192 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 193 => instruction_out <= x"00030780";
when 194 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 195 => instruction_out <= x"c4100780";
when 196 => instruction_out <= x"60040031";   -- IMAD.U16 R12, R0L, R2L, R12;
when 197 => instruction_out <= x"00030780";
when 198 => instruction_out <= x"1c00d201";   -- MOV R0, g [A3+0x9];
when 199 => instruction_out <= x"0423c780";
when 200 => instruction_out <= x"1000e009";   -- MOV R2, g [A4+0x10];
when 201 => instruction_out <= x"0423c784";
when 202 => instruction_out <= x"200b982c";   -- IADD32 R11, R12, R11;
when 203 => instruction_out <= x"40050030";   -- IMUL32.U16.U16 R12, R0L, R2H;
when 204 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 205 => instruction_out <= x"00030780";
when 206 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 207 => instruction_out <= x"c4100780";
when 208 => instruction_out <= x"d8057011";   -- ADA A4, A2, 0x2b8;
when 209 => instruction_out <= x"20000780";
when 210 => instruction_out <= x"60040031";   -- IMAD.U16 R12, R0L, R2L, R12;
when 211 => instruction_out <= x"00030780";
when 212 => instruction_out <= x"1c00d401";   -- MOV R0, g [A3+0xa];
when 213 => instruction_out <= x"0423c780";
when 214 => instruction_out <= x"1000c009";   -- MOV R2, g [A4+0x0];
when 215 => instruction_out <= x"0423c784";
when 216 => instruction_out <= x"200b982c";   -- IADD32 R11, R12, R11;
when 217 => instruction_out <= x"40050030";   -- IMUL32.U16.U16 R12, R0L, R2H;
when 218 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 219 => instruction_out <= x"00030780";
when 220 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 221 => instruction_out <= x"c4100780";
when 222 => instruction_out <= x"60040031";   -- IMAD.U16 R12, R0L, R2L, R12;
when 223 => instruction_out <= x"00030780";
when 224 => instruction_out <= x"1c00d601";   -- MOV R0, g [A3+0xb];
when 225 => instruction_out <= x"0423c780";
when 226 => instruction_out <= x"1000e009";   -- MOV R2, g [A4+0x10];
when 227 => instruction_out <= x"0423c784";
when 228 => instruction_out <= x"200b982c";   -- IADD32 R11, R12, R11;
when 229 => instruction_out <= x"40050030";   -- IMUL32.U16.U16 R12, R0L, R2H;
when 230 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 231 => instruction_out <= x"00030780";
when 232 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 233 => instruction_out <= x"c4100780";
when 234 => instruction_out <= x"d8067011";   -- ADA A4, A2, 0x338;
when 235 => instruction_out <= x"20000780";
when 236 => instruction_out <= x"60040031";   -- IMAD.U16 R12, R0L, R2L, R12;
when 237 => instruction_out <= x"00030780";
when 238 => instruction_out <= x"1c00d801";   -- MOV R0, g [A3+0xc];
when 239 => instruction_out <= x"0423c780";
when 240 => instruction_out <= x"1000c009";   -- MOV R2, g [A4+0x0];
when 241 => instruction_out <= x"0423c784";
when 242 => instruction_out <= x"200b982c";   -- IADD32 R11, R12, R11;
when 243 => instruction_out <= x"40050030";   -- IMUL32.U16.U16 R12, R0L, R2H;
when 244 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 245 => instruction_out <= x"00030780";
when 246 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 247 => instruction_out <= x"c4100780";
when 248 => instruction_out <= x"60040031";   -- IMAD.U16 R12, R0L, R2L, R12;
when 249 => instruction_out <= x"00030780";
when 250 => instruction_out <= x"1c00da01";   -- MOV R0, g [A3+0xd];
when 251 => instruction_out <= x"0423c780";
when 252 => instruction_out <= x"1000e009";   -- MOV R2, g [A4+0x10];
when 253 => instruction_out <= x"0423c784";
when 254 => instruction_out <= x"200b982c";   -- IADD32 R11, R12, R11;
when 255 => instruction_out <= x"40050030";   -- IMUL32.U16.U16 R12, R0L, R2H;
when 256 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 257 => instruction_out <= x"00030780";
when 258 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 259 => instruction_out <= x"c4100780";
when 260 => instruction_out <= x"d8077011";   -- ADA A4, A2, 0x3b8;
when 261 => instruction_out <= x"20000780";
when 262 => instruction_out <= x"60040031";   -- IMAD.U16 R12, R0L, R2L, R12;
when 263 => instruction_out <= x"00030780";
when 264 => instruction_out <= x"1c00dc01";   -- MOV R0, g [A3+0xe];
when 265 => instruction_out <= x"0423c780";
when 266 => instruction_out <= x"1000c009";   -- MOV R2, g [A4+0x0];
when 267 => instruction_out <= x"0423c784";
when 268 => instruction_out <= x"200b982c";   -- IADD32 R11, R12, R11;
when 269 => instruction_out <= x"40050030";   -- IMUL32.U16.U16 R12, R0L, R2H;
when 270 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 271 => instruction_out <= x"00030780";
when 272 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 273 => instruction_out <= x"c4100780";
when 274 => instruction_out <= x"60040031";   -- IMAD.U16 R12, R0L, R2L, R12;
when 275 => instruction_out <= x"00030780";
when 276 => instruction_out <= x"1c00de01";   -- MOV R0, g [A3+0xf];
when 277 => instruction_out <= x"0423c780";
when 278 => instruction_out <= x"1000e009";   -- MOV R2, g [A4+0x10];
when 279 => instruction_out <= x"0423c784";
when 280 => instruction_out <= x"200b982c";   -- IADD32 R11, R12, R11;
when 281 => instruction_out <= x"40050030";   -- IMUL32.U16.U16 R12, R0L, R2H;
when 282 => instruction_out <= x"60040231";   -- IMAD.U16 R12, R0H, R2L, R12;
when 283 => instruction_out <= x"00030780";
when 284 => instruction_out <= x"30101831";   -- SHL R12, R12, 0x10;
when 285 => instruction_out <= x"c4100780";
when 286 => instruction_out <= x"60040001";   -- IMAD.U16 R0, R0L, R2L, R12;
when 287 => instruction_out <= x"00030780";
when 288 => instruction_out <= x"2000002d";   -- IADD R11, R0, R11;
when 289 => instruction_out <= x"0402c780";
when 290 => instruction_out <= x"861ffe03";   -- BAR.ARV.WAIT b0, 0xfff;
when 291 => instruction_out <= x"00000000";
when 292 => instruction_out <= x"20108205";   -- IADD32I R1, R1, 0x10;
when 293 => instruction_out <= x"00000003";
when 294 => instruction_out <= x"30010ffd";   -- ISET.S32.C0 o[0x7f], R7, R1, GE;
when 295 => instruction_out <= x"6c0187c8";
when 296 => instruction_out <= x"20008a15";   -- IADD32I R5, R5, 0x40;
when 297 => instruction_out <= x"00000007";
when 298 => instruction_out <= x"20001011";   -- IADD R4, R8, R4;
when 299 => instruction_out <= x"04010780";
when 300 => instruction_out <= x"10025003";   -- BRA C0.NE, 0x128;
when 301 => instruction_out <= x"00000280";
when 302 => instruction_out <= x"1009e003";   -- BRA  0x4f0;
when 303 => instruction_out <= x"00000780";
when 304 => instruction_out <= x"1000d001";   -- MOV R0, g [0x8];
when 305 => instruction_out <= x"0423c780";
when 306 => instruction_out <= x"40010805";   -- IMUL.U16.U16 R1, R2L, R0H;
when 307 => instruction_out <= x"00000780";
when 308 => instruction_out <= x"60000a05";   -- IMAD.U16 R1, R2H, R0L, R1;
when 309 => instruction_out <= x"00004780";
when 310 => instruction_out <= x"30100205";   -- SHL R1, R1, 0x10;
when 311 => instruction_out <= x"c4100780";
when 312 => instruction_out <= x"60000825";   -- IMAD.U16 R9, R2L, R0L, R1;
when 313 => instruction_out <= x"00004780";
when 314 => instruction_out <= x"1000f82d";   -- MOV R11, R124;
when 315 => instruction_out <= x"0403c780";
when 316 => instruction_out <= x"1000d001";   -- MOV R0, g [0x8];
when 317 => instruction_out <= x"0423c780";
when 318 => instruction_out <= x"40010c05";   -- IMUL.U16.U16 R1, R3L, R0H;
when 319 => instruction_out <= x"00000780";
when 320 => instruction_out <= x"60000e05";   -- IMAD.U16 R1, R3H, R0L, R1;
when 321 => instruction_out <= x"00004780";
when 322 => instruction_out <= x"30100205";   -- SHL R1, R1, 0x10;
when 323 => instruction_out <= x"c4100780";
when 324 => instruction_out <= x"60000c01";   -- IMAD.U16 R0, R3L, R0L, R1;
when 325 => instruction_out <= x"00004780";
when 326 => instruction_out <= x"30040001";   -- SHL R0, R0, 0x4;
when 327 => instruction_out <= x"c4100780";
when 328 => instruction_out <= x"20008c00";   -- IADD32 R0, R6, R0;
when 329 => instruction_out <= x"20098000";   -- IADD32 R0, R0, R9;
when 330 => instruction_out <= x"20001401";   -- IADD R0, R10, R0;
when 331 => instruction_out <= x"04000780";
when 332 => instruction_out <= x"30020001";   -- SHL R0, R0, 0x2;
when 333 => instruction_out <= x"c4100780";
when 334 => instruction_out <= x"2000cc01";   -- IADD R0, g [0x6], R0;
when 335 => instruction_out <= x"04200780";
when 336 => instruction_out <= x"d00e002d";   -- GST.U32 global14[R0], R11;
when 337 => instruction_out <= x"a0c00781";
when 338 => instruction_out <= x"30000003";   -- RET
when 339 => instruction_out <= x"00000780";

			when others => null;
		end case;
	end process;

end arch;

